# from jwtserver.api.v1.routers.reg.phone_status import phone_status
# from jwtserver.api.v1.routers.reg.send_code import call_code
# from jwtserver.api.v1.routers.reg.check_code import check_code
# from jwtserver.api.v1.routers.reg.signup import reg_user
# from jwtserver.api.v1.routers.auth.update_token import update_token
# from jwtserver.api.v1.routers.auth.logout import logout
# from jwtserver.api.v1.routers.auth.login import login
#
